const char vis_join_doc[]="
Merge the given input object into this visual input. The given input becomes invalid and
should not be used afterwards. It is advised to explicitely destroy it with
the 'del' builtin function.

Parameters: (O) : a valid input.
Return:     None
";
static PyObject* vis_join(GGIVisualObject * self, PyObject* args) {
	PyObject * inp;
	
	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"Input invalid");
		return NULL;
	}

	if(!PyArg_ParseTuple(args, "O", &inp)) {
		PyErr_SetString(PyExc_TypeError,"Input required");
		return NULL;
	}
	
	if(!GIIInput_Check(inp)) {
		PyErr_SetString(PyExc_TypeError,"Input required");
		return NULL;
	}
	
	if(((GIIInputObject*)inp)->inp==NULL) {
		PyErr_SetString(PyExc_TypeError,"valid Input required");
		return NULL;
	}

	ggiJoinInputs(self->vis,((GIIInputObject*)args)->inp);
	((GIIInputObject*)inp)->inp=NULL; /* invalidate the object */
	Py_INCREF(Py_None);
	return Py_None;
}

/*------------- Event management  ------------------------*/

const char vis_eventread_doc[]="
Return the next event available on the queue.

Parameters: ()|(i) : the event mask (emAll if not set).
Return:     E: the next event of the given mask, if any
";
PyObject* vis_eventread(GGIVisualObject *self, PyObject *args) {
	int mask;
	gii_event e;
	
	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"Input invalid");
		return NULL;
	}
	
	mask=emAll; /* by default */
	if(!PyArg_ParseTuple(args, "|i", &mask)) {
		return NULL;
	}
	
	if(ggiEventsQueued(self->vis,mask)) {
		ggiEventRead(self->vis, &e, mask);
		return gii_c_api->neweventobjectfromevent(e);		
	}
	Py_INCREF(Py_None);
	return Py_None;
}


const char vis_eventblock_doc[]="
Block until an event in the given mask is available.

Parameters: ()|(i) : the event mask (emAll if not set).
Return:     The mask of available event.
";
PyObject* vis_eventblock(GGIVisualObject *self, PyObject *args) {
	int mask;
	
	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"Input invalid");
		return NULL;
	}
	
	mask=emAll; /* by default */
	if(!PyArg_ParseTuple(args, "|i", &mask)) {
		return NULL;
	}
	
	return PyInt_FromLong(ggiEventPoll(self->vis, mask, NULL));
}

const char vis_eventpoll_doc[]="
Wait for an event in the given mask to be available on the queue.

Parameters: () or one or both (in this order) of 
              - i : the mask (defaults to emAll)
              - [ sec, usec ] : timeout (ints, defaults to 0)
Return:     None : no event occured before timeout
            -i : the mask of available events, if no timeout was specified
            - [ i,[ sec, usec ] ] : same as above but with the remaining time.
";

PyObject* vis_eventpoll(GGIVisualObject *self, PyObject*args) {
	struct timeval t;
	int mask;
	int hastime;

	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"Input invalid");
		return NULL;
	}
	mask=emAll;
	t.tv_usec=0;
	t.tv_sec=0;
	hastime=0;

	switch(PyTuple_Size(args)) {
	case 0:
		break;
	case 1:
		if(PyArg_ParseTuple(args, "i", &mask))
			break;		
		PyErr_Clear();
		if(PyArg_ParseTuple(args, "(ll)", &t.tv_sec,&t.tv_usec)) {
			hastime=1;
			break;
		}
		PyErr_Clear();
		PyErr_SetString(PyExc_TypeError, "Need a event mask and/or a timeout");
		return NULL;
	case 2:
		if(PyArg_ParseTuple(args, "i(ll)",&mask, &t.tv_sec, &t.tv_usec)) {
			hastime=1;
			break;
		}
		return NULL;
	default:
		PyErr_SetString(PyExc_TypeError, "Need at most an event mask and/or a timeout");
		return NULL;
	}
	
	mask=ggiEventPoll(self->vis,mask,&t);
	
	if (mask==0) {
		Py_INCREF(Py_None);
		return Py_None;
	}
	if(hastime)
		return Py_BuildValue("[i[ll]]", mask, t.tv_sec, t.tv_usec);
	return PyInt_FromLong(mask);
}

const char vis_eventsqueued_doc[]="
Return the number of events waiting in the queue.

Parameters: ()|(i) : the event mask i (emAll if not set)
Return:   sure.
";
PyObject* vis_eventsqueued(GGIVisualObject *self, PyObject*args) {
	int mask;
	
	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"not a valid input");
		return NULL;
	}
	
	mask=emAll; /* by default */
	if(!PyArg_ParseTuple(args, "|i", &mask)) {
		return NULL;
	}
	return PyInt_FromLong(ggiEventsQueued(self->vis, mask));
}

const char vis_eventsend_doc[]="
Inject an event given as argument to the input source.
The event must be constructed before.

Parameters: (O) : the event
Return:     None
";
PyObject* vis_eventsend(GGIVisualObject *self, PyObject* args) {
	PyObject * e;

	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"Input invalid");
		return NULL;
	}
	
	if(!PyArg_ParseTuple(args, "O", &e)) {
		PyErr_SetString(PyExc_TypeError, "Event required");
		return NULL;
	}
	
	if(!GIIEvent_Check(e)) {
		PyErr_SetString(PyExc_TypeError,"Event required");
		return NULL;
	}
	
	if(ggiEventSend(self->vis, &(((GIIEventObject*)e)->ev))<0) {
		PyErr_SetString(PyExc_SystemError,"ggiEventSend failed");
		return NULL;
	}
	Py_INCREF(Py_None);
	return Py_None;
}

const char vis_detach_doc[]="
Construct a new Input object with the input source bound to the visual.
Event related functions can't be used on this visual anymore.

Parameters: None
Return:     O : A new input object
";
static PyObject* vis_detach(GGIVisualObject * self) {
	gii_input_t inp;
	GIIInputObject *ret;
	
	if(self->vis->input==NULL) {
		PyErr_SetString(PyExc_TypeError,"not a valid input");
		return NULL;
	}
	
	inp=ggiDetachInput(self->vis);
	ret=PyObject_New(GIIInputObject, gii_c_api->input_type);
	if(!ret) return NULL;
	ret->inp=inp;
	return (PyObject*)ret;
}
